@@ -33,7 +33,7 @@ variable_is_defined(MPEquationParserState *state, const char *name)
     if (strcmp(name, "e") == 0 || strcmp(name, "i") == 0 || strcmp(name, "π") == 0)
         return 1;
     if (state->options->variable_is_defined)
-        return state->options->variable_is_defined(name);
+        return state->options->variable_is_defined(name, state->options->callback_data);
     return 0;
 }
 
@@ -61,7 +61,7 @@ static void
 set_variable(MPEquationParserState *state, const char *name, const MPNumber *x)
 {
     // Reserved words, e, π, mod, and, or, xor, not, abs, log, ln, sqrt, int, frac, sin, cos, ...
-    if (strcmp(name, "e") == 0 || strcmp(name, "π") == 0)
+    if (strcmp(name, "e") == 0 || strcmp(name, "i") == 0 || strcmp(name, "π") == 0)
         return; // FALSE
 
     if (state->options->set_variable)
@@ -118,7 +118,6 @@ static int
 function_is_defined(MPEquationParserState *state, const char *name)
 {
     char *c, *lower_name;
-    int result = 1;
 
     lower_name = strdup(name);
     for (c = lower_name; *c; c++)
@@ -130,8 +129,12 @@ function_is_defined(MPEquationParserState *state, const char *name)
         strcmp(lower_name, "ln") == 0 ||
         strcmp(lower_name, "sqrt") == 0 ||
         strcmp(lower_name, "abs") == 0 ||
+        strcmp(lower_name, "arg") == 0 ||
+        strcmp(lower_name, "conj") == 0 ||
         strcmp(lower_name, "int") == 0 ||
         strcmp(lower_name, "frac") == 0 ||
+        strcmp(lower_name, "re") == 0 ||
+        strcmp(lower_name, "im") == 0 ||
         strcmp(lower_name, "sin") == 0 || strcmp(lower_name, "cos") == 0 || strcmp(lower_name, "tan") == 0 ||
         strcmp(lower_name, "sin⁻¹") == 0 || strcmp(lower_name, "cos⁻¹") == 0 || strcmp(lower_name, "tan⁻¹") == 0 ||
         strcmp(lower_name, "sinh") == 0 || strcmp(lower_name, "cosh") == 0 || strcmp(lower_name, "tanh") == 0 ||
@@ -145,7 +148,7 @@ function_is_defined(MPEquationParserState *state, const char *name)
     g_free (lower_name);
 
     if (state->options->function_is_defined)
-        return state->options->function_is_defined(name);
+        return state->options->function_is_defined(name, state->options->callback_data);
     return 0;
 }
 
@@ -179,10 +182,18 @@ get_function(MPEquationParserState *state, const char *name, const MPNumber *x,
         mp_sqrt(x, z);
     else if (strcmp(lower_name, "abs") == 0) // |x|
         mp_abs(x, z);
+    else if (strcmp(lower_name, "arg") == 0)
+        mp_arg(x, state->options->angle_units, z);
+    else if (strcmp(lower_name, "conj") == 0)
+        mp_conjugate(x, z);
     else if (strcmp(lower_name, "int") == 0)
-        mp_integer_component(x, z);
+        mp_floor(x, z);
     else if (strcmp(lower_name, "frac") == 0)
         mp_fractional_component(x, z);
+    else if (strcmp(lower_name, "re") == 0)
+        mp_real_component(x, z);
+    else if (strcmp(lower_name, "im") == 0)
+        mp_imaginary_component(x, z);
     else if (strcmp(lower_name, "sin") == 0)
         mp_sin(x, state->options->angle_units, z);
     else if (strcmp(lower_name, "cos") == 0)
@@ -235,8 +246,8 @@ do_convert(const char *units[][2], const MPNumber *x, const char *x_units, const
     if (units[z_index][0] == NULL)
         return 0;
 
-    mp_set_from_string(units[x_index][1], &x_factor);
-    mp_set_from_string(units[z_index][1], &z_factor);
+    mp_set_from_string(units[x_index][1], 10, &x_factor);
+    mp_set_from_string(units[z_index][1], 10, &z_factor);
     mp_multiply(x, &x_factor, z);
     mp_divide(z, &z_factor, z);
 
